## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## -----------------------------------------------------------------------------
# library(exams)
# 
# n <- sample(20:30, 1)
# mean_x <- sample(seq(70, 80, by = 0.5), 1)
# choices <- c("t-test", "chi-squared test", "Wilcoxon signed-rank test")
# 
# # Inside the exercise body:
# # How many observations are in the sample? `r add_cloze(n)`
# # What is the sample mean? `r add_cloze(mean_x, tolerance = 0.1, digits = 1)`
# # Which method is appropriate? `r add_cloze(choices[1], choices, type = "schoice")`
# #
# # Meta-information:
# # exclozetype: `r format_metainfo("type")`
# # exsolution: `r format_metainfo("solution")`
# # extol: `r format_metainfo("tolerance")`

## ----questionlist, echo = FALSE, results = "asis"-----------------------------
# exams::answerlist(c(
#   "A p-value below 0.05 can justify rejection at the 5% level.",
#   "A p-value is the probability that the null hypothesis is true.",
#   "A p-value cannot depend on the sample size."
# ), markup = "markdown")

## ----solutionlist, echo = FALSE, results = "asis"-----------------------------
# exams::answerlist(c("True", "False", "False"), markup = "markdown")

